@@ -1,12 +1,11 @@
-predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="none", Df=NULL, lsd_bar=TRUE,
-                          level=0.05, covariate=NULL, meandecr=NULL, trans = I, transOff=0, responsen=NULL, count=FALSE, 
+predictmeans <- function (model, modelterm, data=NULL, pairwise=FALSE, atvar=NULL, adj="none", Df=NULL, lsd_bar=TRUE,
+                          level=NULL, covariate=NULL, meandecr=NULL, letterCI=FALSE, trans = I, transOff=0, responsen=NULL, count=FALSE, 
                           plotord=NULL, plottitle=NULL, plotxlab=NULL, plotylab=NULL, mplot=TRUE, barplot=FALSE, pplot=TRUE, 
-                          bkplot=TRUE, plot=TRUE, jitterv=0.2, basesz=12, prtnum=TRUE, newwd=TRUE,
+                          bkplot=TRUE, plot=TRUE, jitterv=0.2, basesz=12, prtnum=TRUE, prtplt=TRUE, newwd=TRUE,
                           permlist=NULL, ncore=3, ndecimal=4) {
   options(scipen=6)
   if(any(missing(model), missing(modelterm))) stop("The arguments 'model', and 'modelterm' must be provided!")
-  if (!(modelterm %in% attr(terms(model), "term.labels"))) stop(paste("The", modelterm, "must be exactly a term in the model (especially check the order of interaction)."))
-  slevel <- level
+  if (!(modelterm %in% attr(terms(model), "term.labels"))) stop(paste("The", modelterm, "must be exactly a term in the model (especially check the order of interaction)."))  
   predictmeansPlot <- predictmeansBarPlot <- NULL
   if (inherits(model, "aovlist")) stop("Plese use model 'lme' instead of 'aov'!")
   if (inherits(model, "glm")) {
@@ -15,21 +14,27 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
   }
   if (inherits(model, "glmerMod")) {
     trans <- slot(model, "resp")$family$linkinv
-    #    if (slot(model, "resp")$family$family %in% c("poisson", "quasipoisson", "binomial", "quasibinomial")) count=TRUE
+  }
+  if (inherits(model, "glmmTMB")) {
+    trans <- model$modelInfo$family$linkinv
+    if (model$modelInfo$family$family %in% c("poisson", "quasipoisson")) count=TRUE
   }
   vars <- unlist(strsplit(modelterm, "\\:"))
   mdf <- model.frame(model)
   if(any(!is.element(vars, names(mdf)[sapply(mdf,is.factor)])))
     stop(paste(vars, "must be factor(s)!"))
-  
+  # option checking
   if (length(vars)==1) atvar <- NULL
-  if (!is.null(permlist) && !unique(permlist%in%c("NULL", ""))) {pairwise <- TRUE; if (adj=="tukey") stop("The p-value can't be adjusted by Tukey methd!")}                        # option checking
+  if (!is.null(permlist) && !unique(permlist%in%c("NULL", ""))) {pairwise <- TRUE; if (adj=="tukey") stop("The p-value can't be adjusted by Tukey methd!")}
   if (!is.null(atvar) && !unique(atvar%in%c("NULL", ""))) pairwise <- TRUE
   if (adj != "none") pairwise <- TRUE
+  if (letterCI) {atvar <- NULL; pairwise <- TRUE; adj <- "none"; if (is.null(level)) slevel <- level <- 0.166 else slevel <- level}
+  if (is.null(level)) slevel <- level <- 0.05
   if (!is.null(plotord) && !unique(plotord%in%c("NULL", ""))) plot <- mplot <- TRUE
   if (!is.logical(meandecr)) meandecr <- NULL
+  if (!prtplt) newwd <- FALSE
   
-  ctr.matrix <- Kmatrix(model, modelterm, covariate, prtnum=prtnum) 
+  ctr.matrix <- Kmatrix(model, modelterm, covariate, data=data, prtnum=prtnum) 
   KK <- ctr.matrix$K
   label <- ctr.matrix$fctnames
   rownames(label) <- rownames(KK)
@@ -104,11 +109,9 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
                              matrix(unlist(strsplit(KKvarn2, "\\:")), byrow=T, nrow=length(KKvarn2)))
     
     rK <- CM%*%K                    # calculate stats
-	# rK_test <<- rK    # for develop testing
     cm <- rK%*%mp$coef
     vcov.contr <- rK %*% tcrossprod(vcovm, rK)
     dses <- sqrt(diag(vcov.contr))
-    # dses <- as.numeric(apply(rK, 1, function(x) {y <- matrix(x, nrow=1);sqrt(y %*% tcrossprod(vcovm, y))}))
     if (adj == "bonferroni") level <- level/length(dses)
     
     SED.out <- c(Max.SED = max(dses), Min.SED = min(dses), Aveg.SED = mean(dses))
@@ -131,8 +134,8 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
       if (length(Df) == 0) {
         if (inherits(model, "lme")) {
           Df <- terms(model$fixDF)[modelterm]
-        }else if (inherits(model, "merMod")) {
-          Df <- min(df_term(model, modelterm))
+        }else if (inherits(model, "lmerMod")) {
+          Df <- median(df_term(model, modelterm), na.rm=TRUE)
         }else Df <- mp$df
         
         if (Df==0) stop("You need provide Df for this model!")
@@ -173,28 +176,24 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
           vcovm <- x$vcov
           vcov.contr <- rK %*% tcrossprod(vcovm, rK)
           ses <- sqrt(diag(vcov.contr))
-          #ses <- as.numeric(apply(rK, 1, function(x) {y <- matrix(x, nrow=1);sqrt(y %*% tcrossprod(vcovm, y))}))
           t.v <- cm/ses
           return(t.v)	
         }
-		
-        # if (nK==2)	t.p.values <-  (sum(sapply(permlist[[1]], function(x) round(abs(tValue(x, rK)),6) >= round(abs(t.v), 6)))+1)/(nsim+1) else	
-          # t.p.values <-  (rowSums(sapply(permlist[[1]], function(x) round(abs(tValue(x, rK)),6) >= round(abs(t.v), 6)))+1)/(nsim+1)
-		
-    if (.Platform$OS.type=="windows") {
-      cl <- makeCluster(ncore)	  
-      clusterExport(cl, c("tValue", "rK", "t.v"), envir = environment())       
-      t.TableL <- parLapplyLB(cl, permlist[[1]], function(x) round(abs(tValue(x, rK)),6) >= round(abs(t.v), 6)) 
-      stopCluster(cl)
-    }else{  
-      t.TableL <- mclapply(permlist[[1]], function(x) {
-        round(abs(tValue(x, rK)),6) >= round(abs(t.v), 6)
-      }, mc.cores=ncore)
-    }		
-
-	 t.TableL <- matrix(unlist(t.TableL), ncol = length(t.TableL))
-     t.p.values <-  (rowSums(t.TableL)+1)/(nsim+1)	
-		  
+        
+        if (.Platform$OS.type=="windows") {
+          cl <- makeCluster(ncore)	  
+          clusterExport(cl, c("tValue", "rK", "t.v"), envir = environment())       
+          t.TableL <- parLapplyLB(cl, permlist[[1]], function(x) round(abs(tValue(x, rK)),6) >= round(abs(t.v), 6)) 
+          stopCluster(cl)
+        }else{  
+          t.TableL <- mclapply(permlist[[1]], function(x) {
+            round(abs(tValue(x, rK)),6) >= round(abs(t.v), 6)
+          }, mc.cores=ncore)
+        }		
+        
+        t.TableL <- matrix(unlist(t.TableL), ncol = length(t.TableL))
+        t.p.values <-  (rowSums(t.TableL)+1)/(nsim+1)	
+        
       } # end of if (is.null(permlist))
       
       if (is.null(atvar) || all(atvar%in%c("NULL", ""))) {
@@ -225,35 +224,31 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
         t.p.valuemGrp <- t.p.valuem
         t.p.valuemGrp[upper.tri(t.p.valuemGrp)] <- t(t.p.valuemGrp)[upper.tri(t.p.valuemGrp)]
         if (!is.null(groupRn))  t.p.valuemGrp <- t.p.valuemGrp[groupRn, groupRn]
-        # multGrp <- data.frame(multcompLetters(t.p.valuem, Letters=LETTERS, threshold=slevel))
-        # multGrp <- data.frame(multcompLetters(t.p.valuemGrp, Letters=LETTERS, threshold=slevel)[rnK])
-        # names(multGrp) <- "Group"
-        # multGrp <- data.frame(Treatment=groupRn, Mean=mt$pm[order(mt$pm, decreasing = meandecr)], Group=multcompLetters(t.p.valuemGrp, Letters=LETTERS, threshold=slevel)) 
-        # rownames(multGrp) <- NULL
-        
-        # attr(t.p.valuem, paste("Letter-based representation of pairwise comparisons at significant level '", slevel, "'", sep="")) <- multGrp
-        if (all(nrow(t.p.valuep) > 2, pplot, plot)) {
+        if (nrow(t.p.valuep) > 2) p_valueMatrix <- round(t(t.p.valuep), 4)
+        if (all(nrow(t.p.valuep) > 2, pplot, plot, prtplt)) {
           mtitle <- plottitle
           if (is.null(plottitle) || plottitle%in%c("NULL", "")) mtitle <- paste("Level Plot of p-value (adjusted by '", adj, "' method)\n for Pairwise Comparison", sep="")		
-          PMplot(t(t.p.valuep), level=slevel, legendx=0.69, mtitle=mtitle, newwd=newwd)
-          p_valueMatrix <- round(t(t.p.valuep), 4)
-        } # end of if (all(nrow(t.p.valuep) > 2, pplot, plot)) 
+          PMplot(t(t.p.valuep), level=slevel, legendx=0.69, mtitle=mtitle, newwd=newwd)          
+        } 
       }else{
-        # atvar <- vars[which(vars %in% atvar)] # ensure a proper order for atvar
         dses.df$tvalue <- t.v
-        dses.df$pvalue <- t.p.values        
-        dses.df <- merge(dses.df, KKvarndiff, all=TRUE, sort=FALSE)
-        atvar.df <- dses.df  
+        dses.df$pvalue <- t.p.values 
         for (i in which(vars%in%atvar)) { # To find rows relating atvar
-          atvar.df <- atvar.df[which(as.character(atvar.df[, i]) == as.character(atvar.df[, length(vars) + i])), ]
-        }
-        if (adj=="tukey") atvar.df$adj.pvalue <- ptukey(sqrt(2)*abs(atvar.df$tvalue), nK, Df, lower.tail=FALSE) else
-          atvar.df$adj.pvalue <- p.adjust(atvar.df$pvalue, adj)
+          KKvarndiff <- KKvarndiff[which(as.character(KKvarndiff[, i]) == as.character(KKvarndiff[, length(vars) + i])), ]
+        }			
+        
+        atvar.df <- f_loj_krc(KKvarndiff, dses.df, by.x=names(KKvarndiff), by.y=names(KKvarndiff))            
         names(atvar.df)[1:length(vars)] <- vars
-        for (i in vars) {                     # To ensure factor vars  have the same level as original
-          atvar.df[,i] <- factor(atvar.df[,i], levels=levels(mdf[, i][[1]]))
-        }
-        atvar.df <- atvar.df[do.call(order, atvar.df[, atvar, drop=FALSE]),]  # sort data by atvar order
+        atvar.df$adj.pvalue <- unlist(lapply(split(atvar.df, atvar.df[, atvar]), function(x) {
+          t_value <- x$tvalue
+          t_valueN <- length(na.omit(t_value)) 
+          if (t_valueN < 2) x$adj.pvalue <- x$pvalue
+          else{
+            if (adj=="tukey") x$adj.pvalue <- ptukey(sqrt(2)*abs(x$tvalue), t_valueN, Df, lower.tail=FALSE)
+            else x$adj.pvalue <- p.adjust(x$pvalue, adj)
+          }
+        })) 
+        
         
         rnK.df <- as.data.frame(matrix(unlist(strsplit(rnKK, "\\:")), byrow=T, nrow=nKK)) # To find the suitable names for pm
         colnames(rnK.df) <- vars
@@ -269,16 +264,7 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
         
         # mean table arrange by atvar
         mt.atvar <- mt[do.call(order, mt[, atvar, drop=FALSE]),]		
-		bkmt <- mt.atvar[, c(atvar, resvar, "pm", "ses")]
-		
-		# # find decreasing mean order for each level of atvar 
-        # mt.atvar <- mt.atvar[, c(atvar, resvar, "pm")]		
-        # mt.atvar.factor <- factor(do.call("paste", c(mt.atvar[, atvar, drop=FALSE], sep=" : ")))
-		# # print(split(mt.atvar$pm, mt.atvar.factor))
-        # mtlist <- lapply(split(mt.atvar$pm, mt.atvar.factor), function(x) order(x, decreasing = meandecr))
-        # mtlist <- mtlist[atvar.levels]
-       # #  print(mtlist)
-		
+        bkmt <- mt.atvar[, c(atvar, resvar, "pm", "ses")]
         listlength <- length(atvar.levels)
         pmlist <- pmlistTab <- vector("list", listlength)
         indexlength <- nrow(atvar.df)/listlength
@@ -293,21 +279,17 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
           outtab[outtab < 0.0001] <- "0.0001"
           outtab[col(outtab)==row(outtab)] <- 1.0000
           outtab[upper.tri(outtab)] <-""
-         # print(outtab)
           Grpatvar.pm <- t(atvar.pm)+ atvar.pm
-		  # print(Grpatvar.pm)
-          # Grpatvar.pm <- Grpatvar.pm[mtlist[[i]], mtlist[[i]]]
-		  # print(Grpatvar.pm)
           if (all(!is.na(outtab))) outtab <- as.table(cbind(outtab, Group=multcompLetters(Grpatvar.pm, Letters=LETTERS, threshold=slevel)[resvar.levels]))
           pmlistTab[[i]] <- outtab  		  
         }         
         
-        if (all(nrow.pm > 2, pplot, plot)) { 
+        if (nrow.pm > 2) p_valueMatrix <- pmlist
+        if (all(nrow.pm > 2, pplot, plot, prtplt)) { 
           mtitle <- plottitle
           if (is.null(plottitle) || plottitle%in%c("NULL", ""))  mtitle <- paste("Adjusted p-value (by '", adj, 
                                                                                  "' method)\n for Pairwise Comparison at Each Level of '",paste(atvar, collapse =" and "), "'", sep="")
-          PMplot(pmlist, level=slevel, xylabel=rcnplotm, legendx=0.69, mtitle=mtitle, newwd=newwd)
-          p_valueMatrix <- pmlist
+          PMplot(pmlist, level=slevel, xylabel=rcnplotm, legendx=0.69, mtitle=mtitle, newwd=newwd)          
         }    
       } # if (is.null(atvar))
     }# end of if(pairwise)    
@@ -326,17 +308,10 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
     
     rownames(meanTable) <- NULL
     meanTable <- meanTable[c(vars, "pm", "ses", "Df", "LL", "UL")]	
-	meanTable[c("pm", "LL", "UL")] <- round(meanTable[c("pm", "LL", "UL")], ndecimal)
-	meanTable$ses <- round(meanTable$ses, ndecimal+1)
-	
+    meanTable[c("pm", "LL", "UL")] <- round(meanTable[c("pm", "LL", "UL")], ndecimal)
+    meanTable$ses <- round(meanTable$ses, ndecimal+1)    
     colnames(meanTable) <- c(vars, "Mean", "SE", "Df", paste("LL(", (1 - slevel) * 100, "%)", sep = ""),
-                          paste("UL(", (1 - slevel) * 100, "%)", sep = ""))
-    # add group letter to the mean table								
-    # if (pairwise && (is.null(atvar) || atvar%in%c("NULL", "")))	{		
-      # t.p.valuemGrpMT <- t.p.valuem
-      # t.p.valuemGrpMT[upper.tri(t.p.valuemGrpMT)] <- t(t.p.valuemGrpMT)[upper.tri(t.p.valuemGrpMT)]
-      # meanTable$LetterGrp <- multcompLetters(t.p.valuemGrpMT, Letters=LETTERS, threshold=slevel)	
-    # }	  
+                             paste("UL(", (1 - slevel) * 100, "%)", sep = ""))
     
     if (plot) {
       if (length(vars) > 3)
@@ -347,11 +322,11 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
       offSet <- 0.25 * (yMax - yMin)     
       
       if (is.null(atvar) || all(atvar%in%c("NULL", ""))) {
-	    if (lsd_bar) LSD_value <- LSD[3] else LSD_value <- SED.out[3] 	  
+        if (lsd_bar) LSD_value <- LSD[3] else LSD_value <- SED.out[3] 	  
       }else{
-  	    if (lsd_bar)  LSD_value <- mean(attr(LSD,"For the Same Level of Factor")[1, atvar])
-		  else LSD_value <- mean(attr(SED.out, "For the Same Level of Factor")[1, atvar])
-		}
+        if (lsd_bar)  LSD_value <- mean(attr(LSD,"For the Same Level of Factor")[1, atvar])
+        else LSD_value <- mean(attr(SED.out, "For the Same Level of Factor")[1, atvar])
+      }
       if (lsd_bar)  bar_label <- "Aveg.LSD" else bar_label <- "Aveg.SED"
       up <- yMin + LSD_value
       lsdBar <- cbind(plotmt[, vars, drop=FALSE], up=up, yMin=yMin)
@@ -366,30 +341,29 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
           if (newwd) dev.new()
           mtitle <- plottitle
           if (is.null(plottitle) || plottitle%in%c("NULL", "")) mtitle <- paste("Predicted means for \"", vars, "\" with ", bar_label, " (", slevel * 100, "%) Bar", sep="") 
-          p <- qplot(eval(parse(text = vars)), pm, data=plotmt, xlab = mxlab, group=1,
-                     ylab = mylab, main = paste(mtitle, "\n", sep=""), 
-                     ylim = c(yMin - offSet, max(yMax + offSet, yMin + LSD_value + offSet)),
-                     xlim = c(bar_label, levels(plotmt[, vars])))
-          p <- p +geom_point(colour="red")+
-            geom_line(size=0.5)+
-            geom_errorbar(aes(ymax=up, ymin=yMin, x=bar_label), width=0.15, size=0.8, colour="blue") +  #data=lsdBar, 
+          p <- ggplot(plotmt, aes(eval(parse(text = vars)), pm, group=1))+
+            labs(title=paste(mtitle, "\n", sep=""), x=mxlab, y=mylab)+
+            lims(x= c(bar_label, levels(plotmt[, vars])), y = c(yMin - offSet, max(yMax + offSet, yMin + LSD_value + offSet))) +
+            geom_point(colour="red")+
+            geom_line(linewidth=0.5)+
+            geom_errorbar(aes(ymax=up, ymin=yMin, x=bar_label), width=0.15, linewidth=0.8, colour="blue") +  #data=lsdBar, 
             theme_bw(basesz)
           predictmeansPlot <- p
-          print(p)		  
+          if (prtplt) print(p)		  
         } # end of if mplot	   
         if (barplot) {
           if (newwd) dev.new()
           mtitle <- plottitle
           if (is.null(plottitle) || plottitle%in%c("NULL", "")) mtitle <- paste("Predicted means for \"", modelterm, "\" with SE Bars", sep = "")
-          p <- qplot(eval(parse(text = vars)), pm, data=plotmt, ylim = c(min(0, (min(pm) - max(ses)) * 1.2),
-                                                                         max(0, (max(pm) + max(ses)) * 1.2)), xlab=mxlab, ylab=mylab, 
-                     main=paste(mtitle, "\n", sep=""))
           dodge <- position_dodge(width=0.9)
-          p <- p + geom_bar(position=dodge, stat="identity", fill="papayawhip", colour="darkgreen") +
+          p <- ggplot(plotmt, aes(eval(parse(text = vars)), pm))+
+            labs(title=paste(mtitle, "\n", sep=""), x=mxlab, y=mylab)+
+            ylim(c(min(0, (min(pm) - max(ses)) * 1.2), max(0, (max(pm) + max(ses)) * 1.2))) +
+            geom_bar(position=dodge, stat="identity", fill="papayawhip", colour="darkgreen") +
             geom_errorbar(limits, position=dodge, width=0.25, colour="blue")+theme_bw(basesz)+
             theme(panel.border = element_blank(), axis.line = element_line(), panel.grid = element_blank())
           predictmeansBarPlot <- p
-          print(p)
+          if (prtplt) print(p)
         }
       }
       if (length(vars) == 2) {
@@ -410,53 +384,36 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
         
         if (mplot) {     
           if (newwd) dev.new()
-          #		if (is.null(plotord)) plotord <- 1:2
-          #  fact1 <- (vars[plotord])[1]
-          #  fact2 <- (vars[plotord])[2]
-          #  nlvel1 <- nlevels(plotmt[, fact1])
-          #  nlvel2 <- nlevels(plotmt[, fact2])		  
           mtitle <- plottitle
-          if (is.null(plottitle) || plottitle%in%c("NULL", ""))  mtitle <- paste("Predicted means for \"", fact1, "\" by \"", fact2, "\" with ", paste(atvar, collapse=" "), " ", bar_label, " (", 
-                                                                                 slevel * 100, "%) Bar", sep = "")
+          if (is.null(plottitle) || plottitle%in%c("NULL", ""))  mtitle <- paste("Predicted means for \"", fact1, "\" by \"", fact2, "\" with ", paste(atvar, collapse=" "), " ", bar_label, " (", slevel * 100, "%) Bar", sep = "")
           plotmt[, fact1] <- factor(plotmt[, fact1], levels = c(bar_label, levels(plotmt[, fact1])))
-          p <- qplot(eval(parse(text = fact1)), pm, data=plotmt, group=eval(parse(text = fact2)), 
-                     col=eval(parse(text = fact2)), main=paste(mtitle, "\n", sep=""), xlab = mxlab,
-                     ylab = mylab, xlim = levels(plotmt[, fact1]),
-                     ylim = c(yMin - offSet, max(yMax + offSet, yMin + LSD_value + offSet)))
-          p <- p+ geom_line(aes(linetype=eval(parse(text = fact2)), col=eval(parse(text = fact2))), size=0.96)+
-            geom_errorbar(aes(ymax=up, ymin=yMin, x=bar_label), width=0.15, size=0.8, colour="blue")+
+          p <- ggplot(plotmt, aes(eval(parse(text = fact1)), pm, group=eval(parse(text = fact2)), col=eval(parse(text = fact2))))+
+            labs(title=paste(mtitle, "\n", sep=""), x=mxlab, y=mylab)+
+            lims(x= levels(plotmt[, fact1]), y = c(yMin - offSet, max(yMax + offSet, yMin + LSD_value + offSet))) +
+            geom_line(aes(linetype=eval(parse(text = fact2)), col=eval(parse(text = fact2))), linewidth=0.96)+
+            geom_errorbar(aes(ymax=up, ymin=yMin, x=bar_label), width=0.15, linewidth=0.8, colour="blue")+
             guides(linetype = guide_legend(title = fact2))+
             guides(col = guide_legend(title = fact2))+
             theme_bw(basesz)  
           predictmeansPlot <- p
-          print(p)
-          #                  theme(legend.position = c(0.12-0.01*(max(nlvel1-5, 0)), 0.88-0.02*(max(nlvel2-3,0))), 
-          #                    legend.background = element_rect(colour = "black")) )
+          if (prtplt) print(p)
         } # end if mplot
         if (barplot) {
           if (newwd) dev.new()
           mtitle <- plottitle
           if (is.null(plottitle) || plottitle%in%c("NULL", ""))  mtitle <- paste("Predicted means for \"", modelterm, "\" with SE Bars", sep = "")
           dodge <- position_dodge(width=0.9)
-          # p <- qplot(eval(parse(text = fact1)), pm, data=plotmt,
-          # fill=eval(parse(text = fact2)), ylim = c(min(0, (min(pm) - max(ses)) * 1.1),
-          # max(0, (max(pm) + max(ses)) * 1.1)), xlab = mxlab,
-          # ylab = mylab, main = mtitle)+
-          # geom_point(position=dodge) +
-          # geom_bar(stat = "identity", position=dodge)
           p <- ggplot(plotmt, aes(eval(parse(text = fact1)), pm, group=eval(parse(text = fact2)), fill=  eval(parse(text = fact2))))+
             geom_point(position=dodge) +
             geom_bar(stat = "identity", position=dodge)+
             labs( x = mxlab, y = mylab, title =mtitle)+
-            ylim(c(min(0, (min(pm) - max(ses)) * 1.1), max(0, (max(pm) + max(ses)) * 1.1)))	 
-          
-          
-          p <- p + geom_errorbar(limits, position=dodge, width=0.25, colour="blue")+theme_bw(basesz)+
+            ylim(c(min(0, (min(pm) - max(ses)) * 1.1), max(0, (max(pm) + max(ses)) * 1.1))) +
+            geom_errorbar(limits, position=dodge, width=0.25, colour="blue")+theme_bw(basesz)+
             scale_fill_brewer()+
             theme(panel.border = element_blank(), axis.line = element_line(), panel.grid = element_blank())+            
             theme(legend.position = "top")+guides(fill = guide_legend(title = fact2))
           predictmeansBarPlot <- p
-          print(p)
+          if (prtplt) print(p)
         }
       }
       
@@ -479,66 +436,27 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
         mxlab <- plotxlab
         if (is.null(plotxlab) || plotxlab%in%c("NULL", "")) mxlab <- paste("\n", fact1, sep="")
         mylab <- plotylab
-        if (is.null(plotylab) || plotylab%in%c("NULL", "")) mylab <- paste(response, "\n", sep="")        
-        
-        p <- qplot(eval(parse(text = fact1)), pm,  data=plotmt, main=mtitle,
-                   xlab = mxlab, ylab = mylab,
-                   xlim = levels(plotmt[, fact1]), ylim=c(yMin-0.5*offSet, max(yMax+0.5*offSet, yMin+LSD_value+0.5*offSet)),
-                   group=factor(eval(parse(text = fact2))), col=factor(eval(parse(text = fact2)))) +
-          geom_errorbar(aes(ymax=up, ymin=yMin, x=bar_label), width=0.15, size=0.8, colour="blue")+
-          geom_line(aes(linetype=eval(parse(text = fact2)), col=eval(parse(text = fact2))), size=0.8)+
+        if (is.null(plotylab) || plotylab%in%c("NULL", "")) mylab <- paste(response, "\n", sep="") 
+        p <- ggplot(plotmt, aes(eval(parse(text = fact1)), pm, group=factor(eval(parse(text = fact2))), col=factor(eval(parse(text = fact2)))))+
+          labs(title=paste(mtitle, "\n", sep=""), x=mxlab, y=mylab)+
+          lims(x= levels(plotmt[, fact1]), y = c(yMin-0.5*offSet, max(yMax+0.5*offSet, yMin+LSD_value+0.5*offSet))) +
+          geom_errorbar(aes(ymax=up, ymin=yMin, x=bar_label), width=0.15, linewidth=0.8, colour="blue")+
+          geom_line(aes(linetype=eval(parse(text = fact2)), col=eval(parse(text = fact2))), linewidth=0.8)+
           facet_grid(eval(parse(text = paste("~",fact3, sep=""))))+
           guides(group = guide_legend(fact2))+
           guides(linetype = guide_legend(fact2))+
           guides(col = guide_legend(fact2))+
           theme_bw(basesz)
         predictmeansPlot <- p
-        print(p)
+        if (prtplt) print(p)
       }
       
     }
   }
   
-  if (!is.null(trans)) {
-    
+  if (!is.null(trans)) {    
     Mean <- Trt <- predictmeansBKPlot <- NULL
-    # if (identical(trans, make.link("log")$linkinv) || identical(trans, exp)) {
-    # bkmtlog <- t(mapply(bt.log, meanlog=bkmt$pm, sdlog=bkmt$ses, n=rep(round(Df/nrow(bkmt), 0), length(bkmt$pm)), alpha=slevel, SIMPLIFY = T))[, c(1,6,8,9)]
-    # bkmtlog[, c(1,3,4)] <- bkmtlog[, c(1,3,4)]-transOff
-    # bkmt$Mean <- bkmtlog[, "btmean"]
-    # #bkmt$Stder <- bkmtlog[, "sd.mean"]
-    # bkmt$LL <- trans(bkmt$pm - qt(1 - slevel/2, df = Df) * bkmt$ses)-transOff
-    # bkmt$UL <- trans(bkmt$pm + qt(1 - slevel/2, df = Df) * bkmt$ses)-transOff
-    
-    # bkmt$pm <- bkmt$ses <- NULL
-    # nc <- ncol(bkmt)
-    # names(bkmt)[c(nc - 1, nc)] <- c(paste("LL of ", (1 - slevel) * 100, "% CI", sep = ""),
-    # paste("UL of ", (1 - slevel) * 100, "% CI", sep = ""))
-    # bkmt[, (nc - 2):nc] <- round(bkmt[, (nc - 2):nc], 4)
-    # if (count) bkmt[, (nc - 2):nc] <- round(bkmt[, (nc - 2):nc], 0)
-    
-    # }else{
-    
-    
-    # bkmt$Mean <- trans(bkmt$pm)
-    # if (is.null(permlist) || permlist%in%c("NULL", "")) {    
-    # bkmt$LL <- trans(bkmt$pm - qt(1 - slevel/2, df = Df) * bkmt$ses)
-    # bkmt$UL <- trans(bkmt$pm + qt(1 - slevel/2, df = Df) * bkmt$ses)
-    # }else{
-    # bkmt$LL <- trans(bkmt$pm - 2 * bkmt$ses)
-    # bkmt$UL <- trans(bkmt$pm + 2 * bkmt$ses)
-    # }
-    
-    # bkmt$pm <- bkmt$ses <- NULL
-    # nc <- ncol(bkmt)
-    # names(bkmt)[c(nc - 1, nc)] <- c(paste("LL of ", (1 - slevel) * 100, "% CI", sep = ""),
-    # paste("UL of ", (1 - slevel) * 100, "% CI", sep = ""))
-    # bkmt[, (nc - 2):nc] <- round(bkmt[, (nc - 2):nc], 4)
-    # if (count) bkmt[, (nc - 2):nc] <- round(bkmt[, (nc - 2):nc], 0)
-    # }
-    
     bkmt$Mean <- trans(bkmt$pm)-transOff
-    # if (identical(trans, make.link("log")$linkinv) || identical(trans, exp)) bkmt$Mean <- exp(bkmt$pm+bkmt$ses/2)-transOff
     if (identical(trans, make.link("log")$linkinv) || identical(trans, exp)) bkmt$Mean <- exp(bkmt$pm)-transOff
     if (is.null(permlist) || all(permlist%in%c("NULL", ""))) {    
       bkmt$LL <- trans(bkmt$pm - qt(1 - slevel/2, df = Df) * bkmt$ses)-transOff
@@ -549,10 +467,7 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
     }
     
     bkmt$pm <- bkmt$ses <- NULL
-    nc <- ncol(bkmt)
-    # names(bkmt)[c(nc - 1, nc)] <- c(paste("LL of ", (1 - slevel) * 100, "% CI", sep = ""),
-    # paste("UL of ", (1 - slevel) * 100, "% CI", sep = ""))
-    
+    nc <- ncol(bkmt)    
     bkmt[, (nc - 2):nc] <- round(bkmt[, (nc - 2):nc], ndecimal)
     if (count) {
       bkmt[, (nc - 2):nc] <- round(bkmt[, (nc - 2):nc], 0)
@@ -560,14 +475,14 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
     }	
     
     if (plot && bkplot) {
-	
-	  if (!is.null(atvar) && !unique(atvar%in%c("NULL", ""))) mdf <- mdf[do.call(order, mdf[, c(atvar, resvar), drop=FALSE]),]	  
-	  
+      
+      if (!is.null(atvar) && !unique(atvar%in%c("NULL", ""))) mdf <- mdf[do.call(order, mdf[, c(atvar, resvar), drop=FALSE]),]	  
+      
       if (response %in% names(mdf)) {    ## Transformed y before modelling
         if (inherits(mdf[, response], "factor")){
           bky <- as.numeric(mdf[, response])-1
         }else{
-          if (inherits(model, "glm") || inherits(model, "glmerMod")) {
+          if (inherits(model, "glm") || inherits(model, "glmerMod") || inherits(model, "glmmTMB")) {
             bky <- mdf[, response]
             if (!is.null(dim(mdf[, response]))) bky <- mdf[, response][,1]/rowSums(mdf[, response])
           }else{
@@ -577,25 +492,23 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
       }else{       ## Transformed y within modelling
         nresponse <- regmatches(response, regexec("\\(([^<]+)\\)", response))[[1]][2]
         if (!nresponse %in% names(mdf)) {
-          # ques <- paste("R thinks the back transformed response is", sQuote(nresponse), "but the right one is: ")
-          # nresponse <- readline(ques)
           if (is.null(responsen) || all(responsen%in%c("NULL", "")))  stop("Please provide suitable name for response variable using option 'responsen'!")
           nresponse <- responsen
         }
         bky <- mdf[, nresponse]
       }
       if (is.list(bky)) bky <- bky[[1]]	  
-	  
-	 if (is.null(atvar) || all(atvar%in%c("NULL", ""))) { 
-      Trtn <- do.call("paste", c(mdf[, vars, drop=FALSE], sep=":"))  	  
-      newdata2 <- data.frame(bky=bky, Trtn=Trtn)
-      bkmt$Trt <- do.call("paste", c(bkmt[, vars, drop=FALSE], sep=":"))
-	  }else{
-      Trtn <- do.call("paste", c(mdf[, c(atvar, resvar), drop=FALSE], sep=":"))  	  
-      newdata2 <- data.frame(bky=bky, Trtn=Trtn)
-      bkmt$Trt <- do.call("paste", c(bkmt[, c(atvar, resvar), drop=FALSE], sep=":"))
-	  }	  
-	  
+      
+      if (is.null(atvar) || all(atvar%in%c("NULL", ""))) { 
+        Trtn <- do.call("paste", c(mdf[, vars, drop=FALSE], sep=":"))  	  
+        newdata2 <- data.frame(bky=bky, Trtn=Trtn)
+        bkmt$Trt <- do.call("paste", c(bkmt[, vars, drop=FALSE], sep=":"))
+      }else{
+        Trtn <- do.call("paste", c(mdf[, c(atvar, resvar), drop=FALSE], sep=":"))  	  
+        newdata2 <- data.frame(bky=bky, Trtn=Trtn)
+        bkmt$Trt <- do.call("paste", c(bkmt[, c(atvar, resvar), drop=FALSE], sep=":"))
+      }	  
+      
       xMax <- max(max(bkmt[, nc], na.rm=TRUE), bky, na.rm=TRUE)
       xMin <- min(min(bkmt[, nc - 1], na.rm=TRUE), bky, na.rm=TRUE)
       xoffSet <- 0.15 * (xMax - xMin)
@@ -603,97 +516,41 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
       if (is.null(plottitle) || plottitle%in%c("NULL", "")) mtitle <- paste("Back Transformed Means with ", (1 - slevel) * 100, "% CIs\n for '",
                                                                             modelterm, "'", "\n", sep = "")
       if (newwd) dev.new()
-      # limits <- aes(xmax = bkmt$`UL of 95% CI`, xmin=bkmt$`LL of 95% CI`)
       xlimv <- c(xMin - xoffSet, xMax + xoffSet)
-      p <- qplot(Mean, Trt, main = mtitle, ylab = "", xlab = "", xlim = xlimv, data=bkmt)+
-        # geom_point(colour="red") + geom_errorbarh(aes(xmax = `UL of 95% CI`, xmin=`LL of 95% CI`), height=0.2, size=0.8, colour="red") +
-        geom_point(colour="red") + geom_errorbarh(aes(xmax = UL, xmin=LL ), height=0.2, size=0.8, colour="red") +
+      p <- ggplot(bkmt, aes(Mean, Trt))+
+        labs(title=mtitle, x="", y="")+
+        xlim(xlimv) +
+        geom_point(colour="red") + geom_errorbarh(aes(xmax = UL, xmin=LL ), height=0.2, linewidth=0.8, colour="red") +
         scale_y_discrete(limits = rev(unique(bkmt$Trt)))+
         geom_point(aes(x=bky, y=Trtn), shape=1, position = position_jitter(width = jitterv, height = jitterv), colour="blue", alpha=0.6, data=newdata2)+
         theme_bw(basesz)
       predictmeansBKPlot <- p
-      print(p)
+      if (prtplt) print(p)
     }
     rownames(bkmt) <- NULL
     bkmt$Trt <- NULL
     if (!(is.null(atvar) || all(atvar%in%c("NULL", "")))) {
-	  all_var_names <- colnames(meanTable)
+      all_var_names <- colnames(meanTable)
       meanTable <- meanTable[c(atvar, resvar, setdiff(all_var_names, vars))]
       meanTable <- with(meanTable, meanTable[order(eval(parse(text=c(atvar, resvar)[length(vars):1]))),])
-	}
-
-	if (all(!identical(trans, function(x) x, ignore.environment=TRUE), !identical(trans, I, ignore.environment=TRUE))) {
-    meanTable <- cbind(meanTable, round(bkmt[, (ncol(bkmt)-2):ncol(bkmt)], ndecimal))
-    colnames(meanTable)[(ncol(meanTable)-2):ncol(meanTable)] <-  c("Bk_Mean", paste("Bk_LL(", (1 - slevel) * 100, "%)", sep = ""),
-                                                                   paste("Bk_UL(", (1 - slevel) * 100, "%)", sep = ""))
-	}															   
-	
-	if ((is.null(atvar) || all(atvar%in%c("NULL", ""))) && pairwise) {        
-	    if (!is.null(meandecr) && is.logical(meandecr)) meanTable <- meanTable[order(meanTable$Mean, decreasing = meandecr), ]
-		meanTable$LetterGrp <- multcompLetters(t.p.valuemGrp, Letters=LETTERS, threshold=slevel)
-		meanTable <- list(Table=meanTable, Note=paste("Letter-based representation of pairwise comparisons at significant level '", slevel, "'", sep=""))
-	   # attr(meanTable, "Note") <- paste("Note: letter-based representation of pairwise comparisons at significant level '", slevel, "'", sep="")
-	}
+    }
+    
+    if (all(!identical(trans, function(x) x, ignore.environment=TRUE), !identical(trans, I, ignore.environment=TRUE))) {
+      meanTable <- cbind(meanTable, round(bkmt[, (ncol(bkmt)-2):ncol(bkmt)], ndecimal))
+      colnames(meanTable)[(ncol(meanTable)-2):ncol(meanTable)] <-  c("Bk_Mean", paste("Bk_LL(", (1 - slevel) * 100, "%)", sep = ""),
+                                                                     paste("Bk_UL(", (1 - slevel) * 100, "%)", sep = ""))
+    }															   
+    
+    if ((is.null(atvar) || all(atvar%in%c("NULL", ""))) && pairwise) {        
+      if (!is.null(meandecr) && is.logical(meandecr)) meanTable <- meanTable[order(meanTable$Mean, decreasing = meandecr), ]
+      meanTable$LetterGrp <- multcompLetters(t.p.valuemGrp, Letters=LETTERS, threshold=slevel)
+      if (letterCI)  meanTable$LetterGrp <- ci_mcp(meanTable[, grepl("^LL", names(meanTable))], meanTable[, grepl("^UL", names(meanTable))])
+      
+      meanTable <- list(Table=meanTable, Note=paste("Letter-based representation of pairwise comparisons at significant level '", slevel, "'", sep=""))
+      # attr(meanTable, "Note") <- paste("Note: letter-based representation of pairwise comparisons at significant level '", slevel, "'", sep="")
+    }
     predictmeansPlot <- list(predictmeansPlot, predictmeansBKPlot)
-	
-    # if (pairwise) {
-    # if (is.null(atvar) || atvar%in%c("NULL", "")) {
-    # if (all(is.null(permlist) || permlist%in%c("NULL", ""), adj %in% c("none", "bonferroni"))) {
-    # outputlist <- list("Predicted Means" = mean.table, "Standard Error of Means" = se.table,
-    # "Standard Error of Differences" = SED.out, LSD = LSD, "Pairwise LSDs"=round(LSDm, ndecimal+1),
-    # "Pairwise p-values" = round(t.p.valuem,4), "Back Transformed Means" = bkmt, 
-    # predictmeansPlot=predictmeansPlot, predictmeansBKPlot=predictmeansBKPlot, 
-    # predictmeansBarPlot=predictmeansBarPlot, p_valueMatrix=p_valueMatrix, mean_table=meanTable)
-    # class(outputlist) = "pdmlist"
-    # return(outputlist)	
-    # }else{
-    # outputlist <- list("Predicted Means" = mean.table, "Standard Error of Means" = se.table,
-    # "Standard Error of Differences" = SED.out, LSD = LSD,
-    # "Pairwise p-values" = round(t.p.valuem,4), "Back Transformed Means" = bkmt, 
-    # predictmeansPlot=predictmeansPlot, predictmeansBKPlot=predictmeansBKPlot, 
-    # predictmeansBarPlot=predictmeansBarPlot, p_valueMatrix=p_valueMatrix, mean_table=meanTable)
-    # class(outputlist) = "pdmlist"
-    # return(outputlist)	
-    # }
-    # }else{
-    # outputlist <- vector("list", listlength+10)
-    # outputlist[[1]] <- mean.table
-    # outputlist[[2]] <- se.table
-    # outputlist[[3]] <- SED.out
-    # outputlist[[4]] <- LSD		
-    # outputlist[[5]] <- paste("For variable", paste(sQuote(resvar), collapse =" and "), "at each level of", paste(sQuote(atvar), collapse =" and "))                   
-    # for (i in 6: (listlength+5))  outputlist[[i]] <- pmlistTab[[i-5]]
-    # outputlist[[listlength+6]] <- bkmt
-    # outputlist[[listlength+7]] <- predictmeansPlot
-    # outputlist[[listlength+8]] <- predictmeansBKPlot
-    # outputlist[[listlength+9]] <- predictmeansBarPlot
-    # outputlist[[listlength+10]] <- p_valueMatrix
-    # outputlist[[listlength+11]] <- meanTable
-    # if (is.null(permlist) || permlist%in%c("NULL", "")) {
-    # names(outputlist) <- c("Predicted Means", "Standard Error of Means", "Standard Error of Differences",
-    # "LSD", paste("Pairwise comparison p-value (adjusted by", sQuote(adj), "method)"), 
-    # atvar.levels, "Back Transformed Means", "predictmeansPlot", "predictmeansBKPlot", 
-    # "predictmeansBarPlot", "p_valueMatrix", "mean_table")[1:length(outputlist)]												 
-    # }else{
-    # names(outputlist) <- c(c("Predicted Means", "Standard Error of Means", "Standard Error of Differences",
-    # "Approximated LSD"), paste("Pairwise", sQuote(nsim), "times permuted p-value (adjusted by", sQuote(adj), "method)","\n", "for variable", 
-    # paste(sQuote(resvar),collapse =" and "), "at level <", atvar.levels, "> of", paste(sQuote(atvar), collapse =" and ")), 
-    # "Back Transformed Means with an Approximated 95% CIs (Mean +/- 2*SE)", "predictmeansPlot", "predictmeansBKPlot", 
-    # "predictmeansBarPlot", "p_valueMatrix", "mean_table")[1:length(outputlist)]                            
-    # }          
-    # class(outputlist) = "pdmlist"
-    # return(outputlist)	        
-    # }
-    # }else {
-    # outputlist <- list("Predicted Means" = mean.table, "Standard Error of Means" = se.table,
-    # "Standard Error of Differences" = SED.out, LSD = LSD,
-    # "Back Transformed Means" = bkmt, predictmeansPlot=predictmeansPlot, 
-    # predictmeansBKPlot=predictmeansBKPlot, predictmeansBarPlot=predictmeansBarPlot, mean_table=meanTable)
-    # class(outputlist) = "pdmlist"
-    # return(outputlist)	
-    # }
-  } # else {
-  
+  }   
   
   if (pairwise) {
     if (is.null(atvar) || all(atvar%in%c("NULL", ""))) {
@@ -733,12 +590,7 @@ predictmeans <- function (model, modelterm, pairwise=FALSE, atvar=NULL, adj="non
       }else{
         names(outputlist) <- c(c("Predicted Means", "Standard Error of Means", "Standard Error of Differences",
                                  "Approximated LSD"), paste("Pairwise '", nsim, "' times permuted p-value (adjusted by '", adj, "' method)", sep=""), 
-                               atvar.levels,								   
-                               
-                               #paste("Pairwise", sQuote(nsim), "times permuted p-value (adjusted by", sQuote(adj), "method)","\n", "for variable", 
-                               # paste(sQuote(resvar), collapse =" and "), "at level <", atvar.levels, "> of", paste(sQuote(atvar), collapse =" and ")), 
-                               "mean_table", 
-                               "predictmeansPlot", "predictmeansBarPlot", "p_valueMatrix")[1:length(outputlist)]                     
+                               atvar.levels, "mean_table", "predictmeansPlot", "predictmeansBarPlot", "p_valueMatrix")[1:length(outputlist)]                     
       }
       class(outputlist) <- "pdmlist"
       return(outputlist)      
